import java.io.*;
import java.util.AbstractMap.SimpleEntry;

public class IndexSearch {
    public static int searchSLIdx(String key, String[] sLIdx) {
        if (sLIdx == null || key.compareTo(sLIdx[0]) < 0) {
            return -1;
        }
        int indexOfBlock = 0;
        while (indexOfBlock < sLIdx.length) {
            String currentKey = sLIdx[indexOfBlock];
            int comparison = key.compareTo(currentKey);

            // if search key is greater than the current position's key, keep going
            // if search key is equal to the current position's key, return this block
            if (comparison == 0) {
                return indexOfBlock;
            }
            // if search key is less than the current position's key, return the previous block
            if (comparison < 0) {
                return indexOfBlock - 1;
            }
            indexOfBlock++;
        }
        return indexOfBlock - 1;
    }

    public static int getNameFromKey(String key, Object[] pKIdx, String[] sLIdx, int blockSize) {
        int blockIndex = searchSLIdx(key, sLIdx);
        int rowNumber = searchPKIdx(key, pKIdx, blockIndex, blockSize);
        return rowNumber;
    }

    private static int searchPKIdx(String key, Object[] pKIdx, int blockIdx, int blockSize) {
        for (int i = blockIdx * blockSize; i < (blockIdx + 1) * blockSize; i++) {
            if (key.equals(((SimpleEntry<String, Integer>)pKIdx[i]).getKey())) {
                return ((SimpleEntry<String, Integer>) pKIdx[i]).getValue();
            }
        }
        return -1;
    }

    public static void main(String[] args) throws IOException {
        String bookFile = args[0];
        String behavior = args[1];
        int BLOCK_SIZE = 50;
        Object[] entries = IndexBuilder.buildPrimaryKeyIndex(new BufferedReader(new FileReader(bookFile)));

        if (behavior.equals("primary")) {
            for (Object e : entries) {
                System.out.println(e);
            }
        }

        String[] slIdx = IndexBuilder.buildSecondLevelIndex(entries, BLOCK_SIZE);
        if (behavior.equals("secondary")) {
            for (String e : slIdx) {
                System.out.println(e);
            }
        }

        if (behavior.equals("check")) {
            boolean any = false;
            for (Object entry : entries) {
                SimpleEntry<String, Integer> e = (SimpleEntry<String, Integer>) entry;
                int row = getNameFromKey(e.getKey(), entries, slIdx, BLOCK_SIZE);
                if (e.getValue() != row) {
                    any = true;
                    System.out.printf("mismatch: %s, found row number %d", e, row);
                }
            }
            if (!any) {
                System.out.println("All good!");
            }
        }
    }
}
